@@ -14,7 +14,7 @@
  *  Please visit http://freehackers.org/kvim for other vim- or
  *  kde-related coding.
  *
- *  $Id: gui_kde_widget.cc,v 1.55 2002/09/30 16:10:47 marchand Exp $
+ *  $Id: gui_kde_widget.cc,v 1.58 2003/02/22 01:56:31 marchand Exp $
  *
  */
 
@@ -46,6 +46,7 @@
 #include <kstandarddirs.h>
 #endif
 #include "gui_kde_widget.h"
+#include <qxembed.h>
 
 extern "C" {
 #include "version.h"
@@ -111,12 +112,12 @@ struct special_key {//{{{
     { Qt::Key_Help,		'%', '1' },
     //    { Qt::Key_Undo,		'&', '8' }, <= hmmm ?
     { Qt::Key_BackSpace,	'k', 'b' },
-    { Qt::Key_Insert,		'k', 'I' },
-    { Qt::Key_Delete,		'k', 'D' },
-    { Qt::Key_Home,		'k', 'h' },
-    { Qt::Key_End,		'@', '7' },
-    { Qt::Key_Prior,		'k', 'P' },
-    { Qt::Key_Next,		'k', 'N' },
+    { Qt::Key_Insert,		KS_EXTRA, KE_KINS },
+    { Qt::Key_Delete,		KS_EXTRA, KE_KDEL },
+    { Qt::Key_Home,		'K', '1' },
+    { Qt::Key_End,		'K', '4' },
+    { Qt::Key_Prior,		'K', '3' },
+    { Qt::Key_Next,		'K', '5' },
     { Qt::Key_Print,		'%', '9' },
 
     { Qt::Key_Plus,	'K', '6'},
@@ -217,6 +218,7 @@ void VimWidget::mousePressEvent(QMouseEvent *event)//{{{
     stream << kapp->dcopClient()->appId() << button << modifiers << gui.row << gui.col;
     kapp->dcopClient()->emitDCOPSignal("mousePEvent(QCString,int,int,int,int)", params);
 #endif
+    event->accept();
 }//}}}
 
 #if !defined(FEAT_GUI_OPIE) && defined(FEAT_SESSION)
@@ -271,6 +273,7 @@ void VimMainWindow::wheelEvent (QWheelEvent *event)//{{{
     stream << kapp->dcopClient()->appId() << button << modifiers << gui.row << gui.col;
     kapp->dcopClient()->emitDCOPSignal("mouseWhlEvent(QCString, int, int,int,int)", params);
 #endif
+    event->accept();
 }//}}}
 
 void VimWidget::mouseDoubleClickEvent(QMouseEvent *event)//{{{
@@ -303,6 +306,7 @@ void VimWidget::mouseDoubleClickEvent(QMouseEvent *event)//{{{
     stream << kapp->dcopClient()->appId() << button << modifiers << gui.row << gui.col;
     kapp->dcopClient()->emitDCOPSignal("mouseDblClickEvent(QCString, int, int,int,int)", params);
 #endif
+    event->accept();
 }//}}}
 
 void VimWidget::mouseMoveEvent(QMouseEvent *event){//{{{
@@ -344,6 +348,7 @@ void VimWidget::mouseReleaseEvent(QMouseEvent *event)//{{{
 	modifiers|=MOUSE_ALT;
 
     gui_send_mouse_event(MOUSE_RELEASE,event->x(),event->y(),FALSE,modifiers);
+    event->accept();
 }//}}}
 
 /*
@@ -552,7 +557,8 @@ void gui_keypress(QKeyEvent *e) { // {{{
     int modifiers = 0,i;
     uchar string[256],string2[256];
     uchar *s,*d;
-
+    Qt::ButtonState state = e->state();
+    
     QCString unistring = vmw->codec->fromUnicode(e->text());
     if (unistring.length()>0) 
 	strncpy((char*)string, (const char*)unistring,unistring.length());
@@ -591,13 +597,13 @@ void gui_keypress(QKeyEvent *e) { // {{{
 
 
     // change shift-tab (backtab) into S_TAB
-    if ( key == Qt::Key_BackTab && e->state() & Qt::ShiftButton) {
+    if ( key == Qt::Key_BackTab && state & Qt::ShiftButton) {
 	key = Qt::Key_Tab;
     }
 
     // Change C-@ and C-2 in NUL ? Gtk does this
     if ( (key == Qt::Key_2 || key == Qt::Key_At) 
-	    && e->state() & Qt::ControlButton ) {
+	    && state & Qt::ControlButton ) {
 	string[0] = NUL;
 	len = 1;
     } 
@@ -612,10 +618,10 @@ void gui_keypress(QKeyEvent *e) { // {{{
      * that already has the 8th bit set.
      * Don't do this for <S-M-Tab>, that should become K_S_TAB with ALT. */
     if (len == 1
-	    && (e->state() & Qt::AltButton)
-	    && !(key == Qt::Key_BackSpace || key == Qt::Key_Delete)
+	    && (key != Qt::Key_BackSpace && key != Qt::Key_Delete)
 	    && (string[0] & 0x80) == 0
-	    && !(key == Qt::Key_Tab && (e->state() & Qt::ShiftButton)))
+	    && (state & Qt::AltButton)
+	    && !(key == Qt::Key_Tab && (state & Qt::ShiftButton)))
     {
 	string[0] |= 0x80;
 #ifdef FEAT_MBYTE
@@ -638,18 +644,24 @@ void gui_keypress(QKeyEvent *e) { // {{{
     /* Check for special keys, making sure BS and DEL are recognised. */
     if (len == 0 || key == Qt::Key_BackSpace || key == Qt::Key_Delete) 
     {
-	for (i = 0; special_keys[i].qtkey != 0; i++)
+	while (special_keys[i].qtkey != 0 && special_keys[i].qtkey != key ) i++;
+	if (special_keys[i].qtkey != 0) {
+		string[0] = CSI;
+		string[1] = special_keys[i].code0;
+		string[2] = special_keys[i].code1;
+		len = -3;
+	}
+/*
+	for (i = 0; special_keys[i].qtkey != 0 ; i++)
 	{
-	    if (special_keys[i].qtkey == key) 
-	    {
+	    if (special_keys[i].qtkey == key ) {
 		string[0] = CSI;
 		string[1] = special_keys[i].code0;
 		string[2] = special_keys[i].code1;
 		len = -3;
-
 		break;
 	    }
-	}
+	}*/
     }
 
     if (len == 0)   {
@@ -666,9 +678,9 @@ void gui_keypress(QKeyEvent *e) { // {{{
 	    key == Qt::Key_Escape) {
 
 	modifiers = 0;
-	if (e->state() & Qt::ShiftButton) modifiers |= MOD_MASK_SHIFT;
-	if (e->state() & Qt::ControlButton) modifiers |= MOD_MASK_CTRL;
-	if (e->state() & Qt::AltButton) modifiers |= MOD_MASK_ALT;
+	if (state & Qt::ShiftButton) modifiers |= MOD_MASK_SHIFT;
+	if (state & Qt::ControlButton) modifiers |= MOD_MASK_CTRL;
+	if (state & Qt::AltButton) modifiers |= MOD_MASK_ALT;
 
 	/*
 	 * For some keys a shift modifier is translated into another key
@@ -925,8 +937,6 @@ void VimWidget::flash()//{{{
     QObject::connect( repldlg, SIGNAL(replaceAll()), this, SLOT(slotReplaceAll()) );
 #endif
 
-    /* we use a hack based on this assertion, so check it.. */
-    assert(sizeof(void*)==sizeof(int));
 #ifdef FEAT_TOOLBAR
     connect(toolBar(), SIGNAL(clicked(int)), this, SLOT(menu_activated(int)));
 #endif
@@ -945,6 +955,7 @@ void VimWidget::flash()//{{{
         kapp->dcopClient()->registerAs(argServerName->utf8(),false);
 #endif
 #endif
+    QXEmbed::initialize();
 
 }//{{{
 
