@@ -14,7 +14,7 @@
  *  Please visit http://freehackers.org/kvim for other vim- or
  *  kde-related coding.
  *
- *  $Id: gui_kde_x11.cc,v 1.53 2002/09/30 19:19:58 marchand Exp $
+ *  $Id: gui_kde_x11.cc,v 1.55 2003/01/01 17:03:09 marchand Exp $
  *
  */
 
@@ -133,14 +133,15 @@ gui_mch_prepare(int *argc, char **argv)// {{{
 			found=1;
 		}
 #endif
-		if (strcmp(argv[i],"-reverse")==0 ) {
+		if (strcmp(argv[i],"-black")==0 ) {
 			reverse=1;
 			found=1;
 		}
-		if (strcmp(argv[i],"-rv")==0 ) {
+		/* replaced by -black */
+/*		if (strcmp(argv[i],"-rv")==0 ) {
 			reverse=1;
 			found=1;
-		}
+		}*/
 		if (strcmp(argv[i],"-font")==0 || strcmp(argv[i], "-fn")==0 ) {
 			startfont=new QString(argv[i+1]);
 			found=2;
@@ -347,10 +348,6 @@ gui_mch_init()//{{{
 	//gui.menu_height=vmw->menuBar()->height()+1;
 	//gui.toolbar_height=vmw->toolBar()->height();
 
-#if QT_VERSION>=300
-	if (tip==2) KTipDialog::showTip (vmw,QString::null,true);
-	else if (tip==0) KTipDialog::showTip (vmw);
-#endif
 	return OK;
 }//}}}
 
@@ -419,6 +416,11 @@ gui_mch_open()//{{{
 #endif
 	
 	vmw->show();
+#if QT_VERSION>=300
+	if (tip==2) KTipDialog::showTip (vmw,QString::null,true);
+	else if (tip==0) KTipDialog::showTip (vmw);
+#endif
+
 	return OK;
 }//}}}
 
@@ -575,7 +577,7 @@ char_u *gui_mch_font_dialog (char_u *oldval)//{{{
 	n.replace(QRegExp(","),"/");
 	gui.fontname = (char_u *)strdup((const char *)n);
 	n.replace(QRegExp(" "),"\\ ");
-	n=QString("To set this font as your default font for KVim, edit your ~/.gvimrc file and add the following lines : \nif has(\"gui_kde\")\nset guifont=")+n+QString("\nendif \n OR \n use the control center of KDE and choose the correct fixed font");
+	n=QString("To set this font as your default font for KVim, edit your ~/.gvimrc file and add the following lines : \nif has(\"gui_kde\")\nset guifont=")+n+QString("\nendif");// \n OR \n use the control center of KDE and choose the correct fixed font");
 
 	//display a message box which explains how to save your font settings
 #ifndef FEAT_GUI_OPIE
@@ -599,6 +601,7 @@ gui_mch_init_font(char_u * font_name, int fontset)//{{{
 	
 	if (font_name==NULL) {
 #ifndef FEAT_GUI_OPIE
+#if 0
 #if QT_VERSION>=300		
 		KConfig *base = KGlobal::config();
 #else
@@ -616,6 +619,9 @@ gui_mch_init_font(char_u * font_name, int fontset)//{{{
 		QString f = KVimUtils::toString(&ft);
 #endif
 		font_name = (char_u*)strdup(f.latin1()); //latin1 ?
+#else
+		font_name = (char_u*)strdup("misc-fixed/10/-1/5/50/0/0/0/1/0");
+#endif
 #else // FIXME
 		font_name = (char_u*)strdup("");
 #endif
@@ -672,6 +678,7 @@ gui_mch_get_font(char_u * name, int report_error)//{{{
 		return NOFONT; // XFLD names not allowed anymore
 	QFont *myFont = new QFont();
 	fontname.replace(QRegExp("/"),",");
+	myFont->setRawMode(TRUE);
 
 #if QT_VERSION>=300
 	myFont->fromString(fontname);
